@@ -1,5 +1,5 @@
 #!/usr/bin/env python3
-version = "1.2.2"
+version = "1.2.8"
 
 
 # Copyright (C) 2021 by: Justin Douty (jdouty03 at gmail dot com)
@@ -22,7 +22,8 @@ import sys
 import os
 from sys import platform
 import struct
-from math import *
+import math
+from random import random
 import argparse
 import threading
 import time
@@ -38,8 +39,9 @@ warnings.filterwarnings(
 )
 
 # Decide which imports are neccessary based on arguments
-if len(sys.argv) == 1 or (not(sys.argv.count('-o') or sys.argv.count('--export'))):
-  import pyaudio # Don't need pyaudio if just exporting audio as a file
+if len(sys.argv) == 1 or (not(sys.argv.count('-o') or sys.argv.count('--export'))): 
+  # Don't need pyaudio if just exporting audio as a file
+  import pyaudio
   import numpy as np
   
 # You will only need curses if using gui mode
@@ -60,15 +62,16 @@ if len(sys.argv) == 1 or sys.argv.count('--gui') and not(sys.argv.count('-o') or
     curses.KEY_DC = 330
     curses.KEY_ENTER = 10
   elif "linux" in pform:
-    pass # Defaults seem to work fine on linux??
     #curses.KEY_BACKSPACE = 127
     #curses.KEY_DC = 126
-    #curses.KEY_ENTER = 10
+    curses.KEY_ENTER = 10 # Was 343 when tested on Linux Mint (Debian)
   elif "win" in pform:
     curses.KEY_BACKSPACE = 8
   else:
     sys.stderr.write("Warning: unsupported platform, \"" + str(pform) + "\"\n")
-    # Use default curses key definitions; don't change them
+    # Use default curses key definitions; doesn't change them
+
+
 
 
 # A special window that allows typing through characters provided in the type(ch)
@@ -193,8 +196,11 @@ class InputPad:
     text = ""
     for row in range(0, self.ySize):
       self.curPos(0, row)
-      text = text + self.win.instr().decode("utf-8").replace(' ', '')
+      # Remove spaces (which make up the blank space of the curses window)
+      text = text + self.win.instr().decode("utf-8").replace(chr(32), '')
     self.curPos(oldPosX, oldPosY)
+    # Replace the space placeholder character back with spaces
+    text = text.replace(chr(2063), chr(32))
     return text
   
   # Checks if there is an empty slot at the given position
@@ -305,6 +311,19 @@ class threadArgs:
     self.frameSize = 1024
     self.isGUI = False
     self.shutdown = False
+    self.step = 1. # How much to increment x
+    self.functionTable = self.getFunctionTable()
+    
+  # Define functions available to eval here.
+  def getFunctionTable(self):
+    functionsDict = vars(math)
+ 
+    # Returns a random number between -1 and 1. Can be used to generate fuzz noises.
+    def rand():
+      return random() * 2 - 1
+    functionsDict["rand"] = rand
+ 
+    return functionsDict
 
 
 
@@ -324,6 +343,7 @@ class WindowManager:
     # ySize, xSize, yStart, xStart
     self.pad = InputPad(rows - 5, cols, 1, 0)
     
+    
     # Tell the menu that this is the main window
     self.menu.setMainWindow(self.pad.win)
   
@@ -335,7 +355,6 @@ class WindowManager:
     self.menu.setInfoDisplay(self.infoPad) # Give the menu an infoPad
     # so it can display its own information on it.
     
-    
   
     # Set the background and text color of the infoPad
     if curses.has_colors():
@@ -344,7 +363,7 @@ class WindowManager:
     
     self.thread = threading.Thread(target=self.windowThread, args=(tArgs, scr, menu), daemon=True)
     self.thread.start()
-    
+    self.pad.setText(tArgs.expression)
     
   def windowThread(self, tArgs, scr, menu):
     lock = threading.Lock()
@@ -360,21 +379,22 @@ class WindowManager:
         ch = self.scr.getch()
         
         
-        if ch == 27: # Escape key
+        if ch == 27 and menuFocus == False: # Escape key
           with lock:
             self.infoPad.updateInfo("ESC recieved. Shutting down...")
             self.tArgs.shutdown = True
+            break
           
           
-         #Only runs once
+        # Only runs once
         if startUp == True:
           startUp = False
-          self.menu.refreshSides()
+          self.menu.refreshAll()
           self.pad.refresh()
           self.menu.title.refresh()
           
         
-        if ch != -1 and ch != 27:
+        if ch != -1:
           oldTime = time.time()
 
           # Switch between menu and inputPad with the arrow keys
@@ -388,10 +408,10 @@ class WindowManager:
               self.menu.focusedWindow.unhighlight()
             self.infoPad.updateInfo("")
             self.menu.title.refresh() # Good time to refresh the title?
+            self.pad.refresh() # Set cursor back to typing pad
           elif ch == curses.KEY_DOWN:
             menuFocus = True
             settingFocus = True
-            self.menu.getProgressBar().temporaryPause = True
             self.menu.focusedWindow.highlight()
             if self.menu.editing:
               self.menu.focusedWindow.onBeginEdit()
@@ -411,8 +431,7 @@ class WindowManager:
             self.infoPad.updateInfo("Verifying...")
           
             try: # Verify expression
-              eval('lambda x: ' + text)
-              # TODO: better verification and improved "security"...
+              eval('lambda x: ' + text, tArgs.functionTable)
               with lock:
                 self.tArgs.expression = text # Update the expression for the audioThread
               self.infoPad.updateInfo("Playing...")
@@ -477,6 +496,7 @@ class BasicMenuItem:
     self.hoverMsg = "Hover Message"
     self.actionMsg = "Action Message"
     self.displayName = ""
+    self.refresh()
     
  # Calls refresh() on the window object
  # this must be lightweight, as it may be called often.
@@ -552,23 +572,50 @@ class saveButton(InputPad, BasicMenuItem):
   def type(self, ch):
     super().type(ch)
     name = self.getText()
-    path = os.getcwd()
-    fullPath = path + "/" + name + ".wav"
-    if os.path.exists(fullPath):
+    if name == '':
+      self.infoPad.updateInfo(self.toolTip)
+      return
+    
+    # Update the infoDisplay
+    fullPath = ''
+    if '/' in name:
+      # Use absolute specified path
+      fullPath = name + ".wav"
+      parDir = '/'.join(fullPath.split('/')[0:-1])
+      try:
+        if parDir != '' and not os.path.isdir(parDir):
+          self.infoPad.updateInfo("Invalid directory \"" + parDir + "\"")
+          return
+      except PermissionError:
+        self.infoPad.updateInfo("Permission denied: " + parDir)
+    else:
+      path = os.getcwd()
+      fullPath = path + "/" + name + ".wav"
+      
+    if os.path.isfile(fullPath):
       self.infoPad.updateInfo("Will overwrite " + fullPath)
     else:
-      self.infoPad.updateInfo(self.toolTip)
+      self.infoPad.updateInfo("Will export as " + fullPath)
   
   # Where it actually saves the file
   def doAction(self):
     name = self.getText()
-    if name == "":
+    if name == '':
       self.setActionMsg("Cancelled.")
       self.setText("Export Audio")
       return
     self.setText("Export Audio")
-    path = os.getcwd()
-    fullPath = path + "/" + name + ".wav"
+    path = ''
+    if '/' in name:
+      fullPath = name + ".wav"
+      parDir = '/'.join(fullPath.split('/')[0:-1])
+      if not os.path.isdir(parDir):
+        self.setActionMsg("Cannot export audio: invalid path \"" + fullPath + "\"")
+        self.setText("Export Audio")
+        return
+    else:
+      path = os.getcwd()
+      fullPath = path + "/" + name + ".wav"
     self.setActionMsg("Saving file as " + fullPath)
     
     with self.lock:
@@ -591,10 +638,27 @@ def exportAudio(fullPath, tArgs, progressBar, infoPad):
     f.setsampwidth(2)
     f.setframerate(tArgs.rate)
     x = 0
-    exp_as_func = eval('lambda x: ' + "(" + tArgs.expression + ")*32767")
+    exp_as_func = eval('lambda x: ' + "(" + tArgs.expression + ")*32767", tArgs.functionTable)
     # Goes from -32767 to 32767 instead of -1 to 1, and needs to produce ints
     
-    for i in range(tArgs.start, tArgs.end, 1):
+    # Cache values to prevent changes while writing
+    step = tArgs.step
+    start = tArgs.start
+    end = tArgs.end
+    value = 0
+    
+    i=0
+    progressStart = 0 # The point the progress display will start from, so it doesn't go backwards
+    # Support negative (backwards) step
+    if step < 0:
+      i = end
+      progressStart = end
+    elif step > 0:
+      i = start
+      progressStart = start
+    elif step == 0:
+      raise(ValueError("Step value cannot be zero!"))
+    while i <= end and i >= start: # Python's range() does not support using floats as step value
       try:
         value = int(exp_as_func(i))
       except:
@@ -613,8 +677,9 @@ def exportAudio(fullPath, tArgs, progressBar, infoPad):
         oldTime = newTime
         with lock:
           progressBar.temporaryPause = True
-          infoPad.updateInfo("Writing (" + str(int((i-tArgs.start)/(tArgs.end-tArgs.start)*100)) + "%)...")
+          infoPad.updateInfo("Writing (" + str(int(abs(i-progressStart)/(abs(end-start))*100)) + "%)...")
           progressBar.temporaryPause = False
+      i = i + step
   if infoPad:
     infoPad.updateInfo("Exported as " + fullPath)
     
@@ -657,62 +722,113 @@ class NamedMenuSetting(InputPad, BasicMenuItem): # Extend the InputPad and Basic
     super().type(ch)
     self.refresh()
     
-# A special case of a NamedMenuSetting that sets the start range in tArgs
-class startRangeMenuItem(NamedMenuSetting):
-  def __init__(self, ySize, xSize, yStart, xStart, name, tArgs):
+
+      
+# A special case of a NamedMenuSetting that only allows typing numbers. Returns the number as
+# an int or float when getValue() is called based on which one you specified in valueType
+class NumericalMenuSetting(NamedMenuSetting):
+  def __init__(self, ySize, xSize, yStart, xStart, name, type):
     super().__init__(ySize, xSize, yStart, xStart, name)
-    self.lock = threading.Lock()
-    self.tArgs = tArgs
+    self.valueType = "int" # String containing "int" or "float". Defaults to int.
+    self.valueType = type
   
-  def doAction(self):
-    value = str(self.getValue())
+  # Override getValue()
+  def getValue(self):
+    value = str(super().getValue())
     if len(value) == 0:
       value = "0"
     try:
-      value = int(value)
-    except:
-      self.actionMsg = "Error converting string \"" + value + "\" to int."
+      if self.valueType == "int":
+        value = int(value)
+      elif self.valueType == "float":
+        value = float(value)
+    except ValueError:
+      self.actionMsg = "Error converting string \"" + value + "\" to " + self.valueType + "."
+      raise ValueError("Error converting string \"" + value + "\" to " + self.valueType + ".")
     else:
-      self.actionMsg = "Start range changed to " + str(value) + "."
       self.lastValue = str(value)
-      with self.lock:
-        self.tArgs.start = value
-      
-  
-  # Override NamedMenuSetting type method
-  def type(self,ch):
-    # Only allow these keys
-    if (ch >= 48 and ch <= 57) or ch == curses.KEY_BACKSPACE or ch == 127 or ch == curses.KEY_LEFT or ch == curses.KEY_RIGHT or ch == curses.KEY_DC or chr(ch) == '-':
-    # or ch == 260 or ch == 261 or ch == 127 or ch == 45 or ch == 126:
-      super().type(ch)
-      
-# A special case of a NamedMenuSetting that sets the end range in tArgs
-class endRangeMenuItem(NamedMenuSetting):
-  def __init__(self, ySize, xSize, yStart, xStart, name, tArgs):
-    super().__init__(ySize, xSize, yStart, xStart, name)
-    self.lock = threading.Lock()
-    self.tArgs = tArgs
-  
-  def doAction(self):
-    value = str(self.getValue())
-    if len(value) == 0:
-      value = "0"
-    try:
-      value = int(value)
-    except:
-      self.actionMsg = "Error converting string \"" + value + "\" to int."
-    else:
-      self.actionMsg = "End range changed to " + str(value) + "."
-      self.lastValue = str(value)
-      with self.lock:
-        self.tArgs.end = value
-      
-  
-  # Override NamedMenuSetting type method
+      return value
+        
+    # Override NamedMenuSetting type method
   def type(self, ch):
     # Only allow these keys
-    if (ch >= 48 and ch <= 57) or ch == curses.KEY_BACKSPACE or ch == 127 or ch == curses.KEY_LEFT or ch == curses.KEY_RIGHT or ch == curses.KEY_DC or chr(ch) == '-':
-      super().type(ch)
+    if (ch >= 48 and ch <= 57) or ch == curses.KEY_BACKSPACE or ch == 127 or ch == curses.KEY_LEFT or ch == curses.KEY_RIGHT or ch == curses.KEY_DC or chr(ch) == '-' or chr(ch) == '.':
+      if not(chr(ch) == '.' and self.valueType == "int"):
+        super().type(ch)
+
+# A special case of a NamedMenuSetting that sets the start range in tArgs
+class startRangeMenuItem(NumericalMenuSetting):
+  def __init__(self, ySize, xSize, yStart, xStart, name, tArgs):
+    super().__init__(ySize, xSize, yStart, xStart, name, "int")
+    self.lock = threading.Lock()
+    self.tArgs = tArgs
+  
+  def doAction(self):
+    value = 0
+    try:
+      value = self.getValue()
+    except ValueError:
+      pass
+    else:
+      if value > self.tArgs.end: # Don't allow crossing start / end ranges
+        self.actionMsg = "Start range cannot be greater than end range!"
+        self.updateValue(self.tArgs.start)
+      else:
+        self.actionMsg = "Start range changed to " + str(value) + "."
+        self.lastValue = str(value)
+        with self.lock:
+          self.tArgs.start = value
+
+
+# A special case of a NamedMenuSetting that sets the end range in tArgs
+class endRangeMenuItem(NumericalMenuSetting):
+  def __init__(self, ySize, xSize, yStart, xStart, name, tArgs):
+    super().__init__(ySize, xSize, yStart, xStart, name, "int")
+    self.lock = threading.Lock()
+    self.tArgs = tArgs
+    self.stepWin = None
+    
+  # You may give it the stepMenuItem object in order for it to refresh
+  # and update the step if needed.
+  def setStepWin(self, win):
+    self.stepWin = win
+    
+  def doAction(self):
+    value = 0
+    try:
+      value = self.getValue()
+    except ValueError:
+      pass
+    else:
+      if value < self.tArgs.start: # Don't allow crossing start / end ranges
+        self.actionMsg = "End range cannot be less than start range!"
+        self.updateValue(self.tArgs.end)
+      else:
+        self.actionMsg = "End range changed to " + str(value) + "."
+        self.lastValue = str(value)
+        with self.lock:
+          self.tArgs.end = value
+      
+      
+  # A special case of a NamedMenuSetting that sets the step amount in tArgs
+class stepMenuItem(NumericalMenuSetting):
+  def __init__(self, ySize, xSize, yStart, xStart, name, tArgs):
+    super().__init__(ySize, xSize, yStart, xStart, name, "float")
+    self.lock = threading.Lock()
+    self.tArgs = tArgs
+  
+  def doAction(self):
+    value = 0.
+    try:
+      value = self.getValue()
+    except ValueError:
+      pass
+    else:
+      self.actionMsg = "Step amount changed to " + str(value) + ". Note: baud rate is " + str(self.tArgs.rate) + "hz."
+      self.updateValue(value)
+      self.lastValue = str(value)
+      with self.lock:
+        self.tArgs.step = value
     
     
     
@@ -888,10 +1004,11 @@ class UIManager:
     self.boxWidth = int(xSize / 3)
     
     self.title = TitleWindow(1, xSize, 0, 0)
+    self.lock = threading.Lock()
     
-    # Create three windows, side-by-side, two of which are inputPads and are stored in
-    # the settingPads lists
+    # Create windows and store them in the self.settingPads list.
     
+    # Start range
     startWin = startRangeMenuItem(int(ySize/2), self.boxWidth, yStart, xStart, "beg", tArgs)
     startWin.setHoverMsg("Press enter to change the start range.")
     startWin.setToolTip("Setting start range... Press enter to apply.")
@@ -899,42 +1016,53 @@ class UIManager:
     startWin.setDisplayName("Start Value")
     startWin.updateValue(tArgs.start)
     
+    # Progress bar
     progressWin = ProgressBar(int(ySize/2), self.boxWidth, yStart, xStart + self.boxWidth, audioClass, tArgs)
     progressWin.setHoverMsg("Press enter to change the progress bar settings")
     progressWin.setToolTip("V- toggle visibility, Left/Right Arrows- move back and forth, Space- pause/play")
     progressWin.setActionMsg("Done editing.")
     progressWin.setDisplayName("Progress Bar")
     
+    # End range
     endWin = endRangeMenuItem(int(ySize/2), self.boxWidth, yStart, xStart + self.boxWidth * 2, "end", tArgs)
     endWin.setHoverMsg("Press enter to change the end range.")
     endWin.setToolTip("Setting end range... Press enter to apply.")
     endWin.setActionMsg("End range changed!")
     endWin.setDisplayName("End Value")
-    
     endWin.updateValue(tArgs.end)
     
+    # Step value
+    stepWin = stepMenuItem(int(ySize/2), self.boxWidth, yStart + 1, xStart + self.boxWidth * 2, "step", tArgs)
+    stepWin.setHoverMsg("Press enter to change the step amount.")
+    stepWin.setToolTip("Setting step amount... Press enter to apply.")
+    stepWin.setActionMsg("Step amount changed!")
+    stepWin.setDisplayName("Step Value")
+    stepWin.updateValue(tArgs.step)
+    
     # This currently takes up the width of the screen. Change the value from xSize to resize it
-    saveWin = saveButton(int(ySize/2), xSize, yStart+1, xStart, tArgs, progressWin)
+    saveWin = saveButton(int(ySize/2), int(xSize*(2/3)-1), yStart+1, xStart, tArgs, progressWin)
     saveWin.setHoverMsg("Press enter to save a recording as a WAV file.")
     saveWin.setToolTip("Please enter filename, and press enter to save. Any existing file will be overwritten.")
     saveWin.setActionMsg("File saved in same directory!")
-    
     saveWin.setDisplayName("Export Button")
 
     if curses.has_colors():
       curses.init_pair(2, curses.COLOR_RED, -1)
       progressWin.win.bkgd(' ', curses.color_pair(2))
     
+    # Be sure to add your object to the settingPads list!
+    # They will be selected by the arrow keys in the order of this list.
     self.settingPads.append(startWin)
     self.settingPads.append(progressWin)
     self.settingPads.append(endWin)
     self.settingPads.append(saveWin)
+    self.settingPads.append(stepWin)
     
     self.focusedWindow = startWin
     self.focusedWindowIndex = 0
     self.editing = False
-    
-    self.refreshSides()
+      
+    self.refreshAll()
     # End of constructor
     
   # These are "optional" settings that enhance the experience of the menu.
@@ -952,14 +1080,14 @@ class UIManager:
   
   # Calls refresh() for the InputPads for the start and end values,
   # and updates the title window
-  def refreshSides(self):
+  def refreshAll(self):
     self.settingPads[0].updateValue(self.tArgs.start)
     self.settingPads[2].updateValue(self.tArgs.end)
-    self.settingPads[0].refresh()
-    self.settingPads[2].refresh()
     self.settingPads[3].updateValue("Export Audio")
-    self.settingPads[3].refresh()
-    
+    self.settingPads[4].updateValue(self.tArgs.step)
+    for win in self.settingPads:
+      win.refresh()
+  
   # Retrieves the ProgressBar object
   def getProgressBar(self):
     return self.settingPads[1]
@@ -975,6 +1103,20 @@ class UIManager:
     
   # Handles typing and switching between items
   def type(self, ch):
+    # Make it possible to cancel editing using the escape key
+    if ch == 27: # Escape key
+      if self.editing == True:
+        self.editing = False
+        self.focusedWindow.highlight()
+        self.infoPad.updateInfo("Cancelled editing!")
+        self.focusedWindow.highlight()
+        self.refreshTitleMessage()
+        return
+      else: # Not editing? Interpret as shut down
+        with self.lock:
+          self.infoPad.updateInfo("ESC recieved. Shutting down...")
+          self.tArgs.shutdown = True
+    
     # Switch between menu items
     if self.editing == False and (ch == curses.KEY_LEFT or ch == curses.KEY_RIGHT):
       self.focusedWindow.unhighlight()
@@ -996,26 +1138,39 @@ class UIManager:
       
     self.setMainWindow(self.focusedWindow)
     
+    
+    
+    # Function macro to begin edting
+    def beginEdit():
+      self.editing = True
+      self.focusedWindow.unhighlight() # Call the focused item's basic functions
+      self.infoPad.updateInfo(self.focusedWindow.toolTip)
+      self.refreshTitleMessage()
+      self.focusedWindow.onBeginEdit()
+      self.focusedWindow.refresh()
+      
+    # Function macro to end editing
+    def endEdit():
+      self.editing = False
+      # Run action specified in class
+      self.focusedWindow.doAction()
+      self.focusedWindow.highlight()
+      self.infoPad.updateInfo(self.focusedWindow.actionMsg)
+      self.refreshTitleMessage()
+    
     # Toggle editing on and off and handle highlighting
     if ch == curses.KEY_ENTER:
       if self.editing == False:
-        self.editing = True
-        self.focusedWindow.unhighlight()
-        self.infoPad.updateInfo(self.focusedWindow.toolTip)
-        self.refreshTitleMessage()
-        self.focusedWindow.onBeginEdit()
-        self.focusedWindow.refresh()
+        beginEdit()
       else:
-        self.editing = False
-        # Run action specified in class
-        self.focusedWindow.doAction()
-        self.focusedWindow.highlight()
-        self.infoPad.updateInfo(self.focusedWindow.actionMsg)
-        self.refreshTitleMessage()
+        endEdit()
       return
       
+      
     # Drive the type function
     if not (ch == curses.KEY_UP or ch == curses.KEY_DOWN):
+      if self.editing == False:
+        beginEdit() # Begin editing automatically
       self.focusedWindow.type(ch)
     elif self.editing:
       self.infoPad.updateInfo(self.focusedWindow.toolTip)
@@ -1024,6 +1179,7 @@ class UIManager:
 
     
 
+
 # The display at the bottom of the screen that shows status
 # and error messages
 class InfoDisplay:
@@ -1081,37 +1237,40 @@ class AudioPlayer:
     
     value = 0.0
     oldTime = 0
+    frameCount = 0
     
     #Thanks to https://stackoverflow.com/a/12467755 by Ohad
     #for a much more efficient eval() for performance improvements
     
     try: # Don't error-out on an empty text box
-      exp_as_func = eval('lambda x: ' + tArgs.expression)
+      exp_as_func = eval('lambda x: ' + tArgs.expression, tArgs.functionTable)
     except SyntaxError:
       sys.stderr.write("Expression has not yet been set, or other SyntaxError\n")
       pass
       
-      
+    
     try: #Make breaking out of the infinate loop possible by a keyboard interrupt
       #exp_as_func = eval('lambda x: ' + tArgs.expression)
       while tArgs.shutdown is False: # Main loop
         result = []
-        if abs(tArgs.end - tArgs.start < tArgs.frameSize) or tArgs.end <= tArgs.start:
-          sys.stdout.write("Error: Invalid range!")
-          tArgs.shutdown = True
-          break
+        #if abs(tArgs.end - tArgs.start < tArgs.frameSize) or tArgs.end <= tArgs.start:
+        #  sys.stdout.write("Error: Invalid range!")
+        #  tArgs.shutdown = True
+        #  break
         if tArgs.shutdown:
           break
           
         #for self.index in range(tArgs.start, tArgs.end, 1):
-        while self.index <= tArgs.end: # Loop over range
+        while self.index >= tArgs.start and self.index <= tArgs.end: # Loop over range
           try:
             if tArgs.expression:
               value = exp_as_func(self.index) # This is where the waveform values are actually calculated
+              # for an input value, substituted for x
             else:
               value = 0
           except:
             pass
+            
           
           if not(isinstance(value, float)):
             value = 0
@@ -1123,12 +1282,13 @@ class AudioPlayer:
             value = -1
             
           result.append(value)
-          
-          if self.index % tArgs.frameSize == 0 or self.index == 0:
+         # frameCount = frameCount + 1
+          if frameCount >= tArgs.frameSize:
+            frameCount = 0
             if tArgs.shutdown == True:
               sys.exit()
             try: # Set the new expression for next time
-              exp_as_func = eval('lambda x: ' + tArgs.expression)
+              exp_as_func = eval('lambda x: ' + tArgs.expression, tArgs.functionTable)
             except:
               pass
             chunk = np.array(result)
@@ -1147,9 +1307,13 @@ class AudioPlayer:
             if tArgs.shutdown == True:
               self.paused = False
           
-          self.index = self.index + 1
+          self.index = self.index + tArgs.step
+          frameCount = frameCount + 1
+        if tArgs.step < 0: # Allow going backwards
+          self.index = self.tArgs.end
+        else:
+          self.index = self.tArgs.start
           
-        self.index = self.tArgs.start
     except (KeyboardInterrupt, SystemExit):
       pass
     finally:
@@ -1176,7 +1340,7 @@ def main(argv = None):
   parser.add_argument("-e", "--end", type = int, default = 100000,
                       help = "The upper range of x to end at.")
   parser.add_argument("-o", "--export", type = str, default = "", nargs = '?',
-                      help = "Export to specified file as wav and exit")
+                      help = "Export to specified file as wav. File extension is automatically added.")
   parser.add_argument("--channels", type = int, default = 1,
                       help = "The number of audio channels to use")
   parser.add_argument("--rate", type = int, default = 44100,
@@ -1217,9 +1381,12 @@ def main(argv = None):
   window = None
   scr = None
   menu = None
+  
   #The program may be started either in GUI mode or CLI mode. Test for GUI mode vvv
-  if len(sys.argv) == 1 or not(isGuiArgument or isExportArgument or isExpressionProvided):
+  if len(sys.argv) == 1 or isGuiArgument or not(isExportArgument or isExpressionProvided):
     #If no arguments are supplied - GUI
+    if isExpressionProvided:
+      tArgs.expression = args.expression
     sys.stderr.write("Starting GUI mode\n")
     tArgs.isGUI = True
     
@@ -1240,10 +1407,6 @@ def main(argv = None):
     tArgs.isGUI = False
     
     
-    
-  #Start the audio player thread
-  #audio = threading.Thread(target=audioClass, args=(tArgs,), daemon=False)
-  #audio.start()
   
   if len(sys.argv) >= 1 and isExportArgument:
     exportAudio(args.export, tArgs, None, None)
